function init_ground()
    ground = {}
    local l = 0
    for x = 0, 3 do
        table.insert(ground, l + (love.math.random() - 0.5) * 20)
    end
    ground_min = -1
    ground_max = 3
    ground[0] = 0
    ground[-1] = 0
end

function ground_segments_visible()
    local tile_count = love.graphics.getWidth() / 64
    return math.floor(cam.x / 64 - tile_count),
           math.floor(cam.x / 64 + tile_count)
end

function ensure_ground_available(low, high)
    if low < ground_min then
        for newseg = ground_min - 1, low, -1 do
            ground[newseg] = ground[newseg + 1] + (love.math.random() - 0.5) * 20
            maybe_spawn_tank((newseg+1)*64)
            spawn_powerup((newseg+1)*64, ground[newseg] - 80 + love.math.random() * 60)
        end
        ground_min = low
    end

    if high > ground_max then
        for newseg = ground_max + 1, high do
            ground[newseg] = ground[newseg - 1] + (love.math.random() - 0.5) * 20
            maybe_spawn_tank((newseg+1)*64)
            spawn_powerup((newseg+1)*64, ground[newseg] - 80 + love.math.random() * 60)
        end
        ground_max = high
    end
end

function draw_ground()
    love.graphics.setLineStyle('smooth')
    local last = ground[low]
    love.graphics.setColor(255,255,255,255)
    local low, high = ground_segments_visible()

    for i = low+1, high do
        k = ground[i]
        love.graphics.line((i-1)*64+1, last, i*64,k)
        last = k
    end
end

function check_ground_collision(shape)
    for p = 1, #player_shape, 2 do
        local x,y = rotate(player_shape[p], player_shape[p+1], -player.r)
        x = x + player.x
        y = y + player.y

        local gy = ground_height(x)
        if gy < y then
            return p / 2
        end
    end
    return 0
end

function ground_height(x)
    local segment = math.floor(x / 64)
    ensure_ground_available(segment, segment+1)
    local sx_1 = (segment)* 64
    local sx_2 = (segment+1) * 64
    local sy_1 = ground[segment]
    local sy_2 = ground[segment+1]
    local dx = 64
    local dy = sy_2 - sy_1

    return (x-sx_1) / dx * dy + sy_1
end
