require('powerup')
require('bomb')
require('stars')
require('tools')
require('player')
require('tank')
require('ground')
require('post_effects')
require('bullets')
require('debris')
require('fighter')

function love.load()
    sound_coin = love.audio.newSource('coin.ogg', 'static')
    sound_health = love.audio.newSource('health.ogg', 'static')
    sound_gun = love.audio.newSource('gun1.ogg', 'static')
    sound_tank = love.audio.newSource('gun2.ogg', 'static')
    sound_fighter = love.audio.newSource('gun3.ogg', 'static')
    sound_new_fighter = love.audio.newSource('new_fighter.ogg', 'static')
    sound_explosion_bomb = love.audio.newSource('explosion1.ogg', 'static')
    sound_explosion_fighter = love.audio.newSource('explosion2.ogg', 'static')
    sound_explosion_tank = love.audio.newSource('explosion3.ogg', 'static')
    sound_explosion_player = love.audio.newSource('explosion4.ogg', 'static')
    sound_bomb = love.audio.newSource('bombs.ogg', 'static')
    sound_thrust = love.audio.newSource('thrust.ogg', 'static')
    sound_thrust:setLooping(true)
    init_post_effects()
    font = love.graphics.newFont(8)
    reset()
    distortion = true
    distortion_key_was_down = false
end

function bullet_player_collisions()
    for bi = #bullets,1,-1 do
        if bullets[bi].f == 1 then
            local lines = player_shape

            for li = 1, #lines - 2, 2 do
                local x1,y1 = rotate(lines[li], lines[li+1], player.r)
                local x2,y2 = rotate(lines[li+2], lines[li+3], player.r)
                x1 = x1 + player.x
                x2 = x2 + player.x
                y1 = y1 + player.y
                y2 = y2 + player.y

                if line_line_intersection(x1, y1, x2, y2,
                                          bullets[bi].x, bullets[bi].y, bullets[bi].old_x, bullets[bi].old_y) then
                    table.remove(bullets, bi)
                    hurt_player(5)
                    break
                end
            end
        end
    end
end

function bullet_tank_collisions()
    for ti = #tanks,1,-1 do
        for bi = #bullets,1,-1 do
            if bullets[bi].f == 0 then
                local lines = tank_shape
                if tanks[ti].d == 1 then
                    lines = tank_rev_shape
                end

                for li = 1, #lines - 2, 2 do
                    local x1,y1 = rotate(lines[li], lines[li+1], tanks[ti].r)
                    local x2,y2 = rotate(lines[li+2], lines[li+3], tanks[ti].r)
                    x1 = x1 + tanks[ti].x
                    x2 = x2 + tanks[ti].x
                    y1 = y1 + tanks[ti].y
                    y2 = y2 + tanks[ti].y

                    if line_line_intersection(x1, y1, x2, y2,
                                              bullets[bi].x, bullets[bi].y, bullets[bi].old_x, bullets[bi].old_y) then
                        table.remove(bullets, bi)
                        tanks[ti].health = tanks[ti].health - 1
                        break
                    end
                end
            end
        end
    end
end

function bullet_fighter_collisions()
    for ti = #fighters,1,-1 do
        for bi = #bullets,1,-1 do
            if bullets[bi].f == 0 then
                local lines = player_shape

                for li = 1, #lines - 2, 2 do
                    local x1,y1 = rotate(lines[li], lines[li+1], fighters[ti].r)
                    local x2,y2 = rotate(lines[li+2], lines[li+3], fighters[ti].r)
                    x1 = x1 + fighters[ti].x
                    x2 = x2 + fighters[ti].x
                    y1 = y1 + fighters[ti].y
                    y2 = y2 + fighters[ti].y

                    if line_line_intersection(x1, y1, x2, y2,
                                              bullets[bi].x, bullets[bi].y, bullets[bi].old_x, bullets[bi].old_y) then
                        table.remove(bullets, bi)
                        fighters[ti].health = fighters[ti].health - 1
                        break
                    end
                end
            end
        end
    end
end

function ingame_update()

    update_player()
    update_tanks()
    update_fighters()
    update_bullets()
    update_bombs()
    update_powerups()
    cam.y = player.y + cu(0.015*player.vy)
    cam.x = player.x + cu(0.015*player.vx)

    ensure_ground_available(ground_segments_visible())

    bullet_fighter_collisions()
    bullet_tank_collisions()
    if player.health > 0 then
        bullet_player_collisions()
    end

    bomb_rot = bomb_rot + 2* love.timer.getDelta()

    if player.health <= 0 then
        state = 2
    end

end

function love.update()
    update_debris()
    shader:send("time", love.timer.getTime())
    if love.keyboard.isDown('r') then
        reset()
    end

    if state == 0 then
        if love.keyboard.isDown(keys.thrust) then
            state = 1
        end
    elseif state == 1 then
        ingame_update()
    elseif state == 2 then
    end
end

function reset()
    state = 0
    player = {
        x = 0.0,
        y = -50.0,
        vx = 0.0,
        vy = 0.0,
        r = 0.0,
        health = 100.0,
        old_health = 100.0,
        last_bullet = 0
    }

    sound_thrust:stop()

    cam = {
        x = 0.0,
        y = -50.0,
        vx = 0.0,
        vy = 0.0
    }

    init_stars()
    init_ground()
    init_bullets()
    init_tanks()
    init_bombs()
    init_debris()
    init_powerups()
    init_fighters()

    spawn_powerup(50, -50)
    score = 0
    bomb_ammo = 5
    bomb_rot = 0
end

function love.draw()
    local w = love.graphics.getWidth() / 4
    local h = love.graphics.getHeight() / 4
    prepare_post_effect_rendering()

    draw_stars()

    love.graphics.push()
    love.graphics.translate(w / 2 - cam.x,
                            w / 2 - cam.y)
    draw_ground()
    draw_player()
    draw_tanks()
    draw_fighters()
    draw_bullets()
    draw_bombs()
    draw_debris()
    draw_powerups()

    love.graphics.pop()

    love.graphics.setFont(font)
    if state == 0 then
        love.graphics.setColor(255,255,255,255)
        love.graphics.print("[Up]", 20, 20)
            love.graphics.print("Thrust", 60, 20)
        love.graphics.print("[Left]", 20, 30)
            love.graphics.print("Turn Left", 60, 30)
        love.graphics.print("[Right]", 20, 40)
            love.graphics.print("Turn Right", 60, 40)
        love.graphics.print("[X]", 20, 50)
            love.graphics.print("Fire Gun", 60, 50)
        love.graphics.print("[C]", 20, 60)
            love.graphics.print("Drop Bomb", 60, 60)

        local b = (math.sin(love.timer.getTime()*10) * 0.4 + 0.6)*255
        love.graphics.setColor(255,255,255,b)
        love.graphics.printf("Thrust to Start", 0,90,w-1,'center')
    elseif state == 1 then
        love.graphics.setColor(255,255,255,255)
        love.graphics.printf(score, 0,0,w,'right')

        love.graphics.setColor(255-2.55*player.health, player.health, 0, 255)
        love.graphics.line(w/2 - player.health / 2, h - 1, w/2 + player.health / 2, h - 1)
        love.graphics.push()
        love.graphics.translate(5,5)
        love.graphics.rotate(bomb_rot)
        love.graphics.scale(0.75)
        love.graphics.setColor(255,128,0,255)
        love.graphics.line(bomb_shape)
        love.graphics.pop()
        love.graphics.setColor(255,255,255,255)
        love.graphics.print(bomb_ammo, 9, 1)
    elseif state == 2 then
        love.graphics.setColor(255,128,0,255)
        love.graphics.printf("Game Over", 0, h/4, w-1, 'center')
        love.graphics.printf("Press [R] to Restart", 0, h*0.75, w-1, 'center')
        love.graphics.setColor(255,191+math.sin(love.timer.getTime()*10)*64,64*math.sin(love.timer.getTime()*10)+191,255)
        love.graphics.printf(score .. " Points", 0, h/2, w-1, 'center')
    end

    finish_post_effect_rendering()
--    draw_debug_log()
end
