function init_powerups()
    powerups = {}
end

function spawn_powerup(x, y)
    local r = love.math.random()
    local kind = 'coin'
    if r < 0.05 then
        kind = 'bombs'
    elseif r < 0.1 then
        kind = 'health'
    end

    table.insert(powerups, {
        kind = kind,
        x = x,
        y = y 
    })
end

function draw_bomb_powerup()
    love.graphics.rotate(love.timer.getTime() * 3)
    love.graphics.setColor(255,128,0,255)
    love.graphics.line(bomb_shape)
    love.graphics.setColor(0,255,255,255)
    love.graphics.circle('line', 0, 0, 7, 16)
end

function draw_coin()
    love.graphics.setColor(255,0,255,255)
    love.graphics.rotate(love.timer.getTime() * 3)
    love.graphics.scale(0.75, 1)
    love.graphics.circle('line', 0, 0, 2, 16)
    love.graphics.rotate(-love.timer.getTime() * 6)
    love.graphics.scale(0.75, 1)
    love.graphics.setColor(255,128,255,255)
    love.graphics.circle('line', 0, 0, 4, 16)
end

function draw_health()
    local s = math.abs(math.sin(love.timer.getTime() * 3) ) + 3
    love.graphics.scale(s, s)
    love.graphics.setColor(255,255,255,255)
    love.graphics.circle('fill', 0, 0, 2, 16)
    love.graphics.setColor(255,0,0,255)
    love.graphics.line(0, -1, 0, 1)
    love.graphics.line(-1, 0, 1, 0)
end

function draw_powerups()
    for i,p in pairs(powerups) do
        love.graphics.push()
        love.graphics.translate(p.x, p.y)
        if p.kind == 'bombs' then
            draw_bomb_powerup()
        elseif p.kind == 'coin' then
            draw_coin()
        elseif p.kind == 'health' then
            draw_health()
        end
        love.graphics.pop()
    end
end

function update_powerups()
    for u = #powerups, 1, -1 do
        local powerup = powerups[u]
        for p = 1, #player_shape, 2 do
            local x,y = rotate(player_shape[p], player_shape[p+1], -player.r)
            x = x + player.x
            y = y + player.y
            
            local dx = powerup.x - x
            local dy = powerup.y - y

            if dx*dx + dy*dy < 49 then
                if powerup.kind == 'bombs' then
                    sound_bomb:stop()
                    sound_bomb:play()
                    bomb_ammo = bomb_ammo + 5
                elseif powerup.kind == 'coin' then
                    score = score + 5
                    sound_coin:stop()
                    sound_coin:play()
                elseif powerup.kind == 'health' then
                    sound_health:stop()
                    sound_health:play()
                    player.health = math.min(100, player.health + 25)
                end
                table.remove(powerups, u)
                break
            end
        end
    end
end
