tank_shape = {-11,0,11,0,13,-4,8,-8, -11, -3, -11,0}
tank_rev_shape = {-11,0,11,0,11,-3,-8,-8,-13,-4,-11,0}

function draw_tanks()
    for i,t in ipairs(tanks) do
        love.graphics.push()
        love.graphics.translate(t.x,t.y)
        love.graphics.rotate(t.r)
        love.graphics.setColor(255,255,0,255)
        if t.d == 1 then
            love.graphics.line(unpack(tank_rev_shape))
        else
            love.graphics.line(unpack(tank_shape))
        end
        love.graphics.pop()
    end
end

function maybe_spawn_tank(x)
    if love.math.random() > 0.5 then
        table.insert(tanks,
            {x = love.math.random()* 10 - 5 + x, 
             y = 0, 
             r = 0,
             d = 1,
             health = 2,
             last_fire = 0,
             burst_last = 0,
             burst_len = 0})
     end
end

function update_tanks()
    for i = #tanks, 1, -1 do
        if math.abs(tanks[i].x - player.x) < 256 then
            if tanks[i].health <= 0 then
                sound_explosion_tank:stop()
                sound_explosion_tank:play()
                spawn_debris(tanks[i].x, tanks[i].y, {255,255,0})
                table.remove(tanks,i)
                score = score + 10
            else
                if math.abs(tanks[i].x - player.x) > 16 then
                    if tanks[i].x < player.x then
                        tanks[i].d = 1
                    else
                        tanks[i].d = -1
                    end
                end

                tanks[i].x = tanks[i].x + love.timer.getDelta() * 20 * tanks[i].d

                if tanks[i].last_fire < love.timer.getTime() - 4 then
                    if tanks[i].burst_len == 2 then
                        tanks[i].last_fire = love.timer.getTime()
                        tanks[i].burst_len = 0
                    else
                        if tanks[i].burst_last < love.timer.getTime() - 0.5 then
                            sound_tank:stop()
                            sound_tank:play()
                            local r = math.atan2(player.y - tanks[i].y + 20, player.x - tanks[i].x)
                            table.insert(bullets, {x = tanks[i].x,
                                                   y = tanks[i].y-20,
                                                   r = r + math.pi/2,
                                                   vx = math.cos(r) * 200,
                                                   vy = math.sin(r) * 200,
                                                   spawn_time = love.timer.getTime(),
                                                   f = 1})
                            tanks[i].burst_last = love.timer.getTime()
                            tanks[i].burst_len = tanks[i].burst_len + 1
                        end
                    end
                end

                local h1 = ground_height(tanks[i].x)
                local h2 = ground_height(tanks[i].x + 10)
                local h3 = ground_height(tanks[i].x - 10)

                tanks[i].y = h1 - 2
                tanks[i].r = math.atan2(h2-h3,20)
            end
        end
    end
end

function init_tanks()
    tanks = {}
end
